@@ -48,7 +48,7 @@ using ceres::Solve;
 // derivatives.
 struct CostFunctor {
   template <typename T> bool operator()(const T* const x, T* residual) const {
-    residual[0] = T(10.0) - x[0];
+    residual[0] = 10.0 - x[0];
     return true;
   }
 };
