@@ -1,6 +1,6 @@
 // Ceres Solver - A fast non-linear least squares minimizer
-// Copyright 2010, 2011, 2012 Google Inc. All rights reserved.
-// http://code.google.com/p/ceres-solver/
+// Copyright 2017 Google Inc. All rights reserved.
+// http://ceres-solver.org/
 //
 // Redistribution and use in source and binary forms, with or without
 // modification, are permitted provided that the following conditions are met:
@@ -55,7 +55,7 @@ namespace ceres {
 // This class implements the computation of the cost (a.k.a. residual) terms as
 // a function of the input (control) variables, and is the interface for users
 // to describe their least squares problem to Ceres. In other words, this is the
-// modelling layer between users and the Ceres optimizer. The signature of the
+// modeling layer between users and the Ceres optimizer. The signature of the
 // function (number and sizes of input parameter blocks and number of outputs)
 // is stored in parameter_block_sizes_ and num_residuals_ respectively. User
 // code inheriting from this class is expected to set these two members with the
@@ -106,8 +106,7 @@ class CERES_EXPORT CostFunction {
   // the constraints, then returning false whenever the constraints
   // are not satisfied will prevent the solver from moving into the
   // infeasible region. This is not a very sophisticated mechanism for
-  // enforcing constraints, but is often good enough for things like
-  // non-negativity constraints.
+  // enforcing constraints, but is often good enough.
   //
   // Note that it is important that the initial values of the
   // parameter block must be feasible, otherwise the solver will
@@ -116,7 +115,7 @@ class CERES_EXPORT CostFunction {
                         double* residuals,
                         double** jacobians) const = 0;
 
-  const vector<int32>& parameter_block_sizes() const {
+  const std::vector<int32>& parameter_block_sizes() const {
     return parameter_block_sizes_;
   }
 
@@ -125,7 +124,7 @@ class CERES_EXPORT CostFunction {
   }
 
  protected:
-  vector<int32>* mutable_parameter_block_sizes() {
+  std::vector<int32>* mutable_parameter_block_sizes() {
     return &parameter_block_sizes_;
   }
 
@@ -136,7 +135,7 @@ class CERES_EXPORT CostFunction {
  private:
   // Cost function signature metadata: number of inputs & their sizes,
   // number of outputs (residuals).
-  vector<int32> parameter_block_sizes_;
+  std::vector<int32> parameter_block_sizes_;
   int num_residuals_;
   CERES_DISALLOW_COPY_AND_ASSIGN(CostFunction);
 };
