@@ -35,12 +35,13 @@
 #include <string>
 #include <vector>
 #include "ceres/crs_matrix.h"
+#include "ceres/evaluation_callback.h"
+#include "ceres/internal/disable_warnings.h"
 #include "ceres/internal/macros.h"
 #include "ceres/internal/port.h"
 #include "ceres/iteration_callback.h"
 #include "ceres/ordered_groups.h"
 #include "ceres/types.h"
-#include "ceres/internal/disable_warnings.h"
 
 namespace ceres {
 
@@ -118,7 +119,7 @@ class CERES_EXPORT Solver {
   #endif
 #endif
 
-      num_linear_solver_threads = 1;
+      num_linear_solver_threads = -1;
       use_explicit_schur_complement = false;
       use_postordering = false;
       dynamic_sparsity = false;
@@ -136,6 +137,7 @@ class CERES_EXPORT Solver {
       gradient_check_relative_precision = 1e-8;
       gradient_check_numeric_derivative_relative_step_size = 1e-6;
       update_state_every_iteration = false;
+      evaluation_callback = NULL;
     }
 
     // Returns true if the options struct has a valid
@@ -425,9 +427,11 @@ class CERES_EXPORT Solver {
     // whether they are linked into Ceres at build time.
     SparseLinearAlgebraLibraryType sparse_linear_algebra_library_type;
 
-    // Number of threads used by Ceres to solve the Newton
-    // step. Currently only the SPARSE_SCHUR solver is capable of
-    // using this setting.
+    // NOTE: This field is deprecated, and is ignored by
+    // Ceres. Solver::Options::num_threads controls threading for all
+    // of Ceres Solver.
+    //
+    // This setting is scheduled to be removed in 1.15.0.
     int num_linear_solver_threads;
 
     // The order in which variables are eliminated in a linear solver
@@ -738,8 +742,23 @@ class CERES_EXPORT Solver {
     // If true, the user's parameter blocks are updated at the end of
     // every Minimizer iteration, otherwise they are updated when the
     // Minimizer terminates. This is useful if, for example, the user
-    // wishes to visualize the state of the optimization every
-    // iteration.
+    // wishes to visualize the state of the optimization every iteration
+    // (in combination with an IterationCallback).
+    //
+    // NOTE: If an evaluation_callback is provided, then the behaviour
+    // of this flag is slightly different in each case:
+    //
+    // (1) If update_state_every_iteration = false, then the user's
+    // state is changed at every residual and/or jacobian evaluation.
+    // Any user provided IterationCallbacks should NOT inspect and
+    // depend on the user visible state while the solver is running,
+    // since there will be undefined contents.
+    //
+    // (2) If update_state_every_iteration is true, then the user's
+    // state is changed at every residual and/or jacobian evaluation,
+    // BUT the solver will ensure that before the user provided
+    // IterationCallbacks are called, the user visible state will be
+    // updated to the current best point found by the solver.
     bool update_state_every_iteration;
 
     // Callbacks that are executed at the end of each iteration of the
@@ -749,15 +768,28 @@ class CERES_EXPORT Solver {
     // executed.
 
     // Callbacks are executed in the order that they are specified in
-    // this vector. By default, parameter blocks are updated only at
-    // the end of the optimization, i.e when the Minimizer
-    // terminates. This behaviour is controlled by
-    // update_state_every_variable. If the user wishes to have access
-    // to the update parameter blocks when his/her callbacks are
-    // executed, then set update_state_every_iteration to true.
+    // this vector. By default, parameter blocks are updated only at the
+    // end of the optimization, i.e when the Minimizer terminates. This
+    // behaviour is controlled by update_state_every_iteration. If the
+    // user wishes to have access to the updated parameter blocks when
+    // his/her callbacks are executed, then set
+    // update_state_every_iteration to true.
     //
     // The solver does NOT take ownership of these pointers.
     std::vector<IterationCallback*> callbacks;
+
+    // If non-NULL, gets notified when Ceres is about to evaluate the
+    // residuals and/or Jacobians. This enables sharing computation
+    // between residuals, which in some cases is important for efficient
+    // cost evaluation. See evaluation_callback.h for details.
+    //
+    // NOTE: Evaluation callbacks are incompatible with inner iterations.
+    //
+    // WARNING: This interacts with update_state_every_iteration. See
+    // the documentation for that option for more details.
+    //
+    // The solver does NOT take ownership of the pointer.
+    EvaluationCallback* evaluation_callback;
   };
 
   struct CERES_EXPORT Summary {
@@ -841,12 +873,23 @@ class CERES_EXPORT Solver {
     // trust region step.
     double linear_solver_time_in_seconds;
 
+    // Number of times the Newton step was computed by solving a
+    // linear system. This does not include linear solves used by
+    // inner iterations.
+    int num_linear_solves;
+
     // Time (in seconds) spent evaluating the residual vector.
     double residual_evaluation_time_in_seconds;
 
+    // Number of residual only evaluations.
+    int num_residual_evaluations;
+
     // Time (in seconds) spent evaluating the jacobian matrix.
     double jacobian_evaluation_time_in_seconds;
 
+    // Number of Jacobian (and residual) evaluations.
+    int num_jacobian_evaluations;
+
     // Time (in seconds) spent doing inner iterations.
     double inner_iteration_time_in_seconds;
 
@@ -921,10 +964,24 @@ class CERES_EXPORT Solver {
     // num_threads_given if OpenMP is not available.
     int num_threads_used;
 
-    //  Number of threads specified by the user for solving the trust
+    // NOTE: This field is deprecated,
+    // Solver::Summary::num_threads_given should be used instead.
+    //
+    // This field is scheduled to be removed in 1.15.0. In the interim
+    // the value of this field will always be equal to
+    // num_threads_given.
+    //
+    // Number of threads specified by the user for solving the trust
     // region problem.
     int num_linear_solver_threads_given;
 
+    // NOTE: This field is deprecated,
+    // Solver::Summary::num_threads_used should be used instead.
+    //
+    // This field is scheduled to be removed in 1.15.0. In the interim
+    // the value of this field will always be equal to
+    // num_threads_used.
+    //
     // Number of threads actually used by the solver for solving the
     // trust region problem. This number is not equal to
     // num_threads_given if OpenMP is not available.
@@ -1048,9 +1105,8 @@ class CERES_EXPORT Solver {
 };
 
 // Helper function which avoids going through the interface.
-CERES_EXPORT void Solve(const Solver::Options& options,
-           Problem* problem,
-           Solver::Summary* summary);
+CERES_EXPORT void Solve(const Solver::Options& options, Problem* problem,
+                        Solver::Summary* summary);
 
 }  // namespace ceres
 
