@@ -30,8 +30,8 @@
 
 #include "ceres/covariance_impl.h"
 
-#ifdef CERES_USE_OPENMP
-#include <omp.h>
+#if defined(CERES_USE_TBB) || defined(CERES_USE_CXX11_THREADS)
+#include "ceres/parallel_for.h"
 #endif
 
 #include <algorithm>
@@ -52,10 +52,13 @@
 #include "ceres/crs_matrix.h"
 #include "ceres/internal/eigen.h"
 #include "ceres/map_util.h"
+#include "ceres/parallel_utils.h"
 #include "ceres/parameter_block.h"
 #include "ceres/problem_impl.h"
 #include "ceres/residual_block.h"
+#include "ceres/scoped_thread_token.h"
 #include "ceres/suitesparse.h"
+#include "ceres/thread_token_provider.h"
 #include "ceres/wall_time.h"
 #include "glog/logging.h"
 
@@ -75,15 +78,16 @@ CovarianceImpl::CovarianceImpl(const Covariance::Options& options)
     : options_(options),
       is_computed_(false),
       is_valid_(false) {
-#ifndef CERES_USE_OPENMP
+#ifdef CERES_NO_THREADS
   if (options_.num_threads > 1) {
     LOG(WARNING)
-        << "OpenMP support is not compiled into this binary; "
+        << "Neither OpenMP nor TBB support is compiled into this binary; "
         << "only options.num_threads = 1 is supported. Switching "
         << "to single threaded mode.";
     options_.num_threads = 1;
   }
 #endif
+
   evaluate_options_.num_threads = options_.num_threads;
   evaluate_options_.apply_loss_function = options_.apply_loss_function;
 }
@@ -339,49 +343,60 @@ bool CovarianceImpl::GetCovarianceMatrixInTangentOrAmbientSpace(
 
   bool success = true;
 
-// The collapse() directive is only supported in OpenMP 3.0 and higher. OpenMP
-// 3.0 was released in May 2008 (hence the version number).
-#if _OPENMP >= 200805
-#  pragma omp parallel for num_threads(num_threads) schedule(dynamic) collapse(2)
+#if !(defined(CERES_USE_TBB) || defined(CERES_USE_CXX11_THREADS))
+  ThreadTokenProvider thread_token_provider(num_threads);
+#endif // !(defined(CERES_USE_TBB) || defined(CERES_USE_CXX11_THREADS))
+
+  // Technically the following code is a double nested loop where
+  // i = 1:n, j = i:n.
+  int iteration_count = (num_parameters * (num_parameters + 1)) / 2;
+#if defined(CERES_USE_OPENMP)
+#    pragma omp parallel for num_threads(num_threads) schedule(dynamic)
+#endif // CERES_USE_OPENMP
+#if !(defined(CERES_USE_TBB) || defined(CERES_USE_CXX11_THREADS))
+  for (int k = 0; k < iteration_count; ++k) {
 #else
-#  pragma omp parallel for num_threads(num_threads) schedule(dynamic)
-#endif
-  for (int i = 0; i < num_parameters; ++i) {
-    for (int j = 0; j < num_parameters; ++j) {
-      // The second loop can't start from j = i for compatibility with OpenMP
-      // collapse command. The conditional serves as a workaround
-      if (j >= i) {
-        int covariance_row_idx = cum_parameter_size[i];
-        int covariance_col_idx = cum_parameter_size[j];
-        int size_i = parameter_sizes[i];
-        int size_j = parameter_sizes[j];
-#ifdef CERES_USE_OPENMP
-        int thread_id = omp_get_thread_num();
-#else
-        int thread_id = 0;
-#endif
-        double* covariance_block =
-            workspace.get() +
-            thread_id * max_covariance_block_size * max_covariance_block_size;
-        if (!GetCovarianceBlockInTangentOrAmbientSpace(
-                parameters[i], parameters[j], lift_covariance_to_ambient_space,
-                covariance_block)) {
-          success = false;
-        }
+  problem_->context()->EnsureMinimumThreads(num_threads);
+  ParallelFor(problem_->context(),
+              0,
+              iteration_count,
+              num_threads,
+              [&](int thread_id, int k) {
+#endif // !(defined(CERES_USE_TBB) || defined(CERES_USE_CXX11_THREADS))
+      int i, j;
+      LinearIndexToUpperTriangularIndex(k, num_parameters, &i, &j);
 
-        covariance.block(covariance_row_idx, covariance_col_idx,
-                         size_i, size_j) =
-            MatrixRef(covariance_block, size_i, size_j);
+      int covariance_row_idx = cum_parameter_size[i];
+      int covariance_col_idx = cum_parameter_size[j];
+      int size_i = parameter_sizes[i];
+      int size_j = parameter_sizes[j];
+#if !(defined(CERES_USE_TBB) || defined(CERES_USE_CXX11_THREADS))
+      const ScopedThreadToken scoped_thread_token(&thread_token_provider);
+      const int thread_id = scoped_thread_token.token();
+#endif // !(defined(CERES_USE_TBB) || defined(CERES_USE_CXX11_THREADS))
+      double* covariance_block =
+          workspace.get() +
+          thread_id * max_covariance_block_size * max_covariance_block_size;
+      if (!GetCovarianceBlockInTangentOrAmbientSpace(
+              parameters[i], parameters[j], lift_covariance_to_ambient_space,
+              covariance_block)) {
+        success = false;
+      }
 
-        if (i != j) {
-          covariance.block(covariance_col_idx, covariance_row_idx,
-                           size_j, size_i) =
-              MatrixRef(covariance_block, size_i, size_j).transpose();
+      covariance.block(covariance_row_idx, covariance_col_idx,
+                       size_i, size_j) =
+          MatrixRef(covariance_block, size_i, size_j);
+
+      if (i != j) {
+        covariance.block(covariance_col_idx, covariance_row_idx,
+                         size_j, size_i) =
+            MatrixRef(covariance_block, size_i, size_j).transpose();
 
-        }
       }
     }
-  }
+#if defined(CERES_USE_TBB) || defined(CERES_USE_CXX11_THREADS)
+   );
+#endif // defined(CERES_USE_TBB) || defined(CERES_USE_CXX11_THREADS)
   return success;
 }
 
@@ -696,33 +711,50 @@ bool CovarianceImpl::ComputeCovarianceValuesUsingSuiteSparseQR() {
   const int num_threads = options_.num_threads;
   scoped_array<double> workspace(new double[num_threads * num_cols]);
 
+#if !(defined(CERES_USE_TBB) || defined(CERES_USE_CXX11_THREADS))
+  ThreadTokenProvider thread_token_provider(num_threads);
+#endif // !(defined(CERES_USE_TBB) || defined(CERES_USE_CXX11_THREADS))
+
+#ifdef CERES_USE_OPENMP
 #pragma omp parallel for num_threads(num_threads) schedule(dynamic)
+#endif // CERES_USE_OPENMP
+
+#if !(defined(CERES_USE_TBB) || defined(CERES_USE_CXX11_THREADS))
   for (int r = 0; r < num_cols; ++r) {
+#else
+  problem_->context()->EnsureMinimumThreads(num_threads);
+  ParallelFor(problem_->context(),
+              0,
+              num_cols,
+              num_threads,
+              [&](int thread_id, int r) {
+#endif // !(defined(CERES_USE_TBB) || defined(CERES_USE_CXX11_THREADS))
+
     const int row_begin = rows[r];
     const int row_end = rows[r + 1];
-    if (row_end == row_begin) {
-      continue;
-    }
+    if (row_end != row_begin) {
+#if !(defined(CERES_USE_TBB) || defined(CERES_USE_CXX11_THREADS))
+      const ScopedThreadToken scoped_thread_token(&thread_token_provider);
+      const int thread_id = scoped_thread_token.token();
+#endif // !(defined(CERES_USE_TBB) || defined(CERES_USE_CXX11_THREADS))
 
-#  ifdef CERES_USE_OPENMP
-    int thread_id = omp_get_thread_num();
-#  else
-    int thread_id = 0;
-#  endif
-
-    double* solution = workspace.get() + thread_id * num_cols;
-    SolveRTRWithSparseRHS<SuiteSparse_long>(
-        num_cols,
-        static_cast<SuiteSparse_long*>(R->i),
-        static_cast<SuiteSparse_long*>(R->p),
-        static_cast<double*>(R->x),
-        inverse_permutation[r],
-        solution);
-    for (int idx = row_begin; idx < row_end; ++idx) {
-     const int c = cols[idx];
-     values[idx] = solution[inverse_permutation[c]];
+      double* solution = workspace.get() + thread_id * num_cols;
+      SolveRTRWithSparseRHS<SuiteSparse_long>(
+          num_cols,
+          static_cast<SuiteSparse_long*>(R->i),
+          static_cast<SuiteSparse_long*>(R->p),
+          static_cast<double*>(R->x),
+          inverse_permutation[r],
+          solution);
+      for (int idx = row_begin; idx < row_end; ++idx) {
+        const int c = cols[idx];
+        values[idx] = solution[inverse_permutation[c]];
+      }
     }
   }
+#if defined(CERES_USE_TBB) || defined(CERES_USE_CXX11_THREADS)
+  );
+#endif // defined(CERES_USE_TBB) || defined(CERES_USE_CXX11_THREADS)
 
   free(permutation);
   cholmod_l_free_sparse(&R, &cc);
@@ -888,37 +920,55 @@ bool CovarianceImpl::ComputeCovarianceValuesUsingEigenSparseQR() {
   const int num_threads = options_.num_threads;
   scoped_array<double> workspace(new double[num_threads * num_cols]);
 
+#if !(defined(CERES_USE_TBB) || defined(CERES_USE_CXX11_THREADS))
+  ThreadTokenProvider thread_token_provider(num_threads);
+#endif // !(defined(CERES_USE_TBB) || defined(CERES_USE_CXX11_THREADS))
+
+#ifdef CERES_USE_OPENMP
 #pragma omp parallel for num_threads(num_threads) schedule(dynamic)
+#endif // CERES_USE_OPENMP
+
+#if !(defined(CERES_USE_TBB) || defined(CERES_USE_CXX11_THREADS))
   for (int r = 0; r < num_cols; ++r) {
+#else
+  problem_->context()->EnsureMinimumThreads(num_threads);
+  ParallelFor(problem_->context(),
+              0,
+              num_cols,
+              num_threads,
+              [&](int thread_id, int r) {
+#endif // !(defined(CERES_USE_TBB) || defined(CERES_USE_CXX11_THREADS))
+
     const int row_begin = rows[r];
     const int row_end = rows[r + 1];
-    if (row_end == row_begin) {
-      continue;
-    }
+    if (row_end != row_begin) {
+#if !(defined(CERES_USE_TBB) || defined(CERES_USE_CXX11_THREADS))
+      const ScopedThreadToken scoped_thread_token(&thread_token_provider);
+      const int thread_id = scoped_thread_token.token();
+#endif // !(defined(CERES_USE_TBB) || defined(CERES_USE_CXX11_THREADS))
 
-#  ifdef CERES_USE_OPENMP
-    int thread_id = omp_get_thread_num();
-#  else
-    int thread_id = 0;
-#  endif
+      double* solution = workspace.get() + thread_id * num_cols;
+      SolveRTRWithSparseRHS<int>(
+          num_cols,
+          qr_solver.matrixR().innerIndexPtr(),
+          qr_solver.matrixR().outerIndexPtr(),
+          &qr_solver.matrixR().data().value(0),
+          inverse_permutation.indices().coeff(r),
+          solution);
 
-    double* solution = workspace.get() + thread_id * num_cols;
-    SolveRTRWithSparseRHS<int>(
-        num_cols,
-        qr_solver.matrixR().innerIndexPtr(),
-        qr_solver.matrixR().outerIndexPtr(),
-        &qr_solver.matrixR().data().value(0),
-        inverse_permutation.indices().coeff(r),
-        solution);
-
-    // Assign the values of the computed covariance using the
-    // inverse permutation used in the QR factorization.
-    for (int idx = row_begin; idx < row_end; ++idx) {
-     const int c = cols[idx];
-     values[idx] = solution[inverse_permutation.indices().coeff(c)];
+      // Assign the values of the computed covariance using the
+      // inverse permutation used in the QR factorization.
+      for (int idx = row_begin; idx < row_end; ++idx) {
+       const int c = cols[idx];
+       values[idx] = solution[inverse_permutation.indices().coeff(c)];
+      }
     }
   }
 
+#if defined(CERES_USE_TBB) || defined(CERES_USE_CXX11_THREADS)
+  );
+#endif // defined(CERES_USE_TBB) || defined(CERES_USE_CXX11_THREADS)
+
   event_logger.AddEvent("Inverse");
 
   return true;
