@@ -45,7 +45,6 @@
 #include "ceres/types.h"
 #include "glog/logging.h"
 #include "gmock/gmock.h"
-#include "gmock/mock-log.h"
 #include "gtest/gtest.h"
 
 namespace ceres {
@@ -55,7 +54,6 @@ using std::vector;
 using testing::AllOf;
 using testing::AnyNumber;
 using testing::HasSubstr;
-using testing::ScopedMockLog;
 using testing::_;
 
 // Pick a (non-quadratic) function whose derivative are easy:
@@ -162,11 +160,12 @@ TEST(GradientCheckingCostFunction, ResidualsAndJacobiansArePreservedTest) {
   const double kRelativePrecision = 1e-4;
 
   TestTerm<-1, -1> term(arity, dim);
+  GradientCheckingIterationCallback callback;
   scoped_ptr<CostFunction> gradient_checking_cost_function(
-      CreateGradientCheckingCostFunction(&term,
+      CreateGradientCheckingCostFunction(&term, NULL,
                                          kRelativeStepSize,
                                          kRelativePrecision,
-                                         "Ignored."));
+                                         "Ignored.", &callback));
   term.Evaluate(&parameters[0],
                 &original_residual,
                 &original_jacobians[0]);
@@ -218,39 +217,35 @@ TEST(GradientCheckingCostFunction, SmokeTest) {
   LOG(INFO) << "Bad gradient";
   {
     TestTerm<1, 2> term(arity, dim);
+    GradientCheckingIterationCallback callback;
     scoped_ptr<CostFunction> gradient_checking_cost_function(
-        CreateGradientCheckingCostFunction(&term,
+        CreateGradientCheckingCostFunction(&term, NULL,
                                            kRelativeStepSize,
                                            kRelativePrecision,
-                                           "Fuzzy bananas"));
-
-    ScopedMockLog log;
-    EXPECT_CALL(log, Log(_, _, _)).Times(AnyNumber());
-    EXPECT_CALL(log, Log(WARNING, _,
-                         AllOf(HasSubstr("(1,0,2) Relative error worse than"),
-                               HasSubstr("Fuzzy bananas"))));
-
-    gradient_checking_cost_function->Evaluate(&parameters[0],
-                                              &residual,
-                                              &jacobians[0]);
+                                           "Fuzzy banana", &callback));
+    EXPECT_TRUE(
+        gradient_checking_cost_function->Evaluate(&parameters[0], &residual,
+                                                  &jacobians[0]));
+    EXPECT_TRUE(callback.gradient_error_detected());
+    EXPECT_TRUE(callback.error_log().find("Fuzzy banana") != std::string::npos);
+    EXPECT_TRUE(callback.error_log().find("(1,0,2) Relative error worse than")
+                != std::string::npos);
   }
 
   // The gradient is correct, so no errors are reported.
   LOG(INFO) << "Good gradient";
   {
     TestTerm<-1, -1> term(arity, dim);
+    GradientCheckingIterationCallback callback;
     scoped_ptr<CostFunction> gradient_checking_cost_function(
-        CreateGradientCheckingCostFunction(&term,
+        CreateGradientCheckingCostFunction(&term, NULL,
                                            kRelativeStepSize,
                                            kRelativePrecision,
-                                           "Ignored."));
-
-    ScopedMockLog log;
-    EXPECT_CALL(log, Log(_, _, _)).Times(0);
-
-    gradient_checking_cost_function->Evaluate(&parameters[0],
-                                              &residual,
-                                              &jacobians[0]);
+                                           "Fuzzy banana", &callback));
+    EXPECT_TRUE(
+        gradient_checking_cost_function->Evaluate(&parameters[0], &residual,
+                                                  &jacobians[0]));
+    EXPECT_FALSE(callback.gradient_error_detected());
   }
 
   for (int j = 0; j < arity; j++) {
@@ -362,8 +357,9 @@ TEST(GradientCheckingProblemImpl, ProblemDimensionsMatch) {
   problem_impl.AddResidualBlock(new TernaryCostFunction(1, 5, 3, 4),
                                 NULL, z, x, y);
 
+  GradientCheckingIterationCallback callback;
   scoped_ptr<ProblemImpl> gradient_checking_problem_impl(
-      CreateGradientCheckingProblemImpl(&problem_impl, 1.0, 1.0));
+      CreateGradientCheckingProblemImpl(&problem_impl, 1.0, 1.0, &callback));
 
   // The dimensions of the two problems match.
   EXPECT_EQ(problem_impl.NumParameterBlocks(),
@@ -383,7 +379,7 @@ TEST(GradientCheckingProblemImpl, ProblemDimensionsMatch) {
   // Since we added the ParameterBlocks and ResidualBlocks explicitly,
   // they should be in the same order in the two programs. It is
   // possible that may change due to implementation changes to
-  // Program. This is not exepected to be the case and writing code to
+  // Program. This is not expected to be the case and writing code to
   // anticipate that possibility not worth the extra complexity in
   // this test.
   for (int i = 0; i < program.parameter_blocks().size(); ++i) {
