@@ -28,11 +28,6 @@
 //
 // Author: kushalav@google.com (Avanish Kushal)
 
-// This include must come before any #ifndef check on Ceres compile options.
-#include "ceres/internal/port.h"
-
-#ifndef CERES_NO_SUITESPARSE
-
 #include "ceres/visibility.h"
 
 #include <cmath>
@@ -162,5 +157,3 @@ WeightedGraph<int>* CreateSchurComplementGraph(
 
 }  // namespace internal
 }  // namespace ceres
-
-#endif  // CERES_NO_SUITESPARSE
